﻿using System;
using System.Windows;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Input.StylusPlugIns;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 잉크 컨트롤
    /// </summary>
    public class InkControl : Label
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 잉크 프리젠터
        /// </summary>
        private InkPresenter presenter;

        /// <summary>
        /// 동적 렌더러
        /// </summary>
        private DynamicRenderer renderer;

        /// <summary>
        /// 스타일러스 포인트 컬렉션
        /// </summary>
        private StylusPointCollection pointCollection = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - InkControl()

        /// <summary>
        /// 생성자
        /// </summary>
        static InkControl()
        {
            Type ownerType = typeof(InkControl);

            ClipToBoundsProperty.OverrideMetadata(ownerType, new FrameworkPropertyMetadata(true));
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - InkControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public InkControl()
        {
            this.presenter = new InkPresenter();

            Content = this.presenter;

            this.renderer = new DynamicRenderer();

            this.presenter.AttachVisuals(this.renderer.RootVisual, this.renderer.DrawingAttributes);

            StylusPlugIns.Add(this.renderer);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 스타일러스 DOWN 처리하기 - OnStylusDown(e)

        /// <summary>
        /// 스타일러스 DOWN 처리하기
        /// </summary>
        /// <param name="e">이벤트 발생자</param>
        protected override void OnStylusDown(StylusDownEventArgs e)
        {
            Stylus.Capture(this);

            this.pointCollection = new StylusPointCollection();

            StylusPointCollection pointCollection = e.GetStylusPoints(this, this.pointCollection.Description);

            this.pointCollection.Add(pointCollection);
        }

        #endregion
        #region 스타일러스 이동시 처리하기 - OnStylusMove(e)

        /// <summary>
        /// 스타일러스 이동시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnStylusMove(StylusEventArgs e)
        {
            if(this.pointCollection == null)
            {
                return;
            }

            StylusPointCollection pointCollection = e.GetStylusPoints(this, this.pointCollection.Description);

            this.pointCollection.Add(pointCollection);
        }

        #endregion
        #region 스타일러스 UP 처리하기 - OnStylusUp(e)

        /// <summary>
        /// 스타일러스 UP 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnStylusUp(StylusEventArgs e)
        {
            if(this.pointCollection == null)
            {
                return;
            }

            StylusPointCollection pointCollection = e.GetStylusPoints(this, this.pointCollection.Description);

            this.pointCollection.Add(pointCollection);

            Stroke stroke = new Stroke(this.pointCollection);

            this.presenter.Strokes.Add(stroke);

            this.pointCollection = null;

            Stylus.Capture(null);
        }

        #endregion

        #region 마우슨 왼쪽 버튼 DOWN 처리하기 - OnMouseLeftButtonDown(e)

        /// <summary>
        /// 마우슨 왼쪽 버튼 DOWN 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);

            if(e.StylusDevice != null)
            {
                return;
            }

            this.pointCollection = new StylusPointCollection();

            Point point = e.GetPosition(this);

            this.pointCollection.Add(new StylusPoint(point.X, point.Y));
        }

        #endregion
        #region 마우스 이동시 처리하기 - OnMouseMove(e)

        /// <summary>
        /// 마우스 이동시 처리하기
        /// </summary>
        /// <param name="e">이벤트 발생자</param>
        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);

            if(e.StylusDevice != null)
            {
                return;
            }

            if(e.LeftButton == MouseButtonState.Released || this.pointCollection == null)
            {
                return;
            }

            Point point = e.GetPosition(this);

            this.pointCollection.Add(new StylusPoint(point.X, point.Y));
        }

        #endregion
        #region 마우스 왼쪽 버튼 UP 처리하기 - OnMouseLeftButtonUp(e)

        /// <summary>
        /// 마우스 왼쪽 버튼 UP 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonUp(e);

            if(e.StylusDevice != null)
            {
                return;
            }

            if(this.pointCollection == null)
            {
                return;
            }

            Point point = e.GetPosition(this);

            this.pointCollection.Add(new StylusPoint(point.X, point.Y));

            Stroke stroke = new Stroke(this.pointCollection);

            stroke.DrawingAttributes = this.renderer.DrawingAttributes;

            this.presenter.Strokes.Add(stroke);

            this.pointCollection = null;
        }

        #endregion
    }
}